﻿#!/usr/bin/python
# -*- coding: utf-8 -*-

import re
from QonsoExtractor import QonsoExtractor
from PyQt4 import QtWebKit, QtCore
from PyQt4.QtNetwork import QNetworkRequest
from DebugLog import DebugLog

class QonsoExtractorSFR(QonsoExtractor):
   def __init__(self):
      DebugLog("QonsoExtractorSFR::init")
      
      QonsoExtractor.__init__(self)
      
      # URL de la page d'info conso
      self.urlconso = "https://monprofil.sfr.fr/monprofilWEBV5/Accueil?acces=enc"
      
      # URL présentant le formulaire pour la connection au compte
      self.urlbeforeAuth = "https://www.sfr.fr/cas/login?domain=mon-compte"
      
      # URL où on envoie la requête POST pour l'authentification, en général, elle est dans le form 
      self.urlAuth = "https://www.sfr.fr/cas/login?domain=mon-compte&service=https%3A%2F%2Fwww.sfr.fr%2Fj_spring_cas_security_check"
      
      self.hostname = "www.sfr.fr"
      
   def Authenticate(self):
      DebugLog("QonsoExtractorSFR::Authenticate")
      self.SetStatus('INFO', 'Authentification...')

      params = QtCore.QUrl()
      # Mettre ici les paramètres du formulaire (login, password, champs cachés)
      params.addQueryItem("username", self.login)
      params.addQueryItem("password", self.password)
      params.addQueryItem("internal", "W")
      #
      
      self.PostRequest(params)
      
   def DoReadResponse(self, reply):
      DebugLog("QonsoExtractorSFR::DoReadResponse")
      
      request = QNetworkRequest()
      page = QtWebKit.QWebPage()
      frame = page.currentFrame()

      # Phase d'initialisation, on va extraire 'idCorrelation', et on lance l'authentification avec
      DebugLog("authentication=="+self.authentication)
      
      if self.authentication=='Init':
         self.authentication = 'InProgress'
         self.Authenticate()
      
      # Authentification en cours, on récupère la page de redirection, et on fait une redirection
      elif self.authentication == 'InProgress':
         html = str( reply.readAll() )
         
         if not self.CheckForErrors(html):
            # pas d'erreur, c'est Lesieur, on peut charger la page des consommations
            request.setUrl( QtCore.QUrl(self.urlconso) ) 
            self.authentication = 'Done'
            self.manager.get(request)
      
   def CheckForErrors(self, datas):
      r = re.compile('<p class="error">\s*<span>(.+?)<.*?</p>', re.DOTALL)
      res = r.search(datas)
      
      if res:
         DebugLog("Erreur trouvée : "+str(res.groups()) )

         error = res.group(1).strip()
         if error:
            self.SetStatus('ERROR', error)
            return True
         else:
            DebugLog("Pas d'erreur en fait...")
            return False
      return False

   # Extraction des données, dépend de chaque opérateur
   def DoExtractDatas(self, datas):
      DebugLog("QonsoExtractorSFR::DoExtractDatas")

      if self.CheckForErrors(datas):
         return
      
      # Voix
      DebugLog("\n_______________________________________________\nVoix\n")
      DebugLog("-----------------------------------------------\n")
      r = re.compile('vous reste %s de communication .* votre [Ff]orfait (.+?)\s*<b>%s' % (self.hmReS, self.hmReS), re.DOTALL)
      res = r.search(datas)
      if res:
         self.model.SetConsoVoixMax( res.group(3) )
         self.model.SetConsoVoix( res.group(1), invert=True )
         self.model.SetForfait( res.group(2) )
         DebugLog("voix2 = "+res.group(1))
         DebugLog("voixMax2 = "+res.group(3))
         DebugLog("forfait2 = "+res.group(2))
      else:
         r = re.compile('votre [Ff]orfait.*?"(.*?)".*?[Vv]ous avez un [Ff]orfait .+?%s et vous avez consomm.*?%s de communication'%(self.hmReS,self.hmReS), re.DOTALL)
         res = r.search(datas)
         if res:
            self.model.SetForfait( res.group(1) )
            self.model.SetConsoVoixMax( res.group(2) )
            self.model.SetConsoVoix( res.group(3) )
            DebugLog("forfait = "+res.group(1))
            DebugLog("voix1 = "+res.group(1))
            DebugLog("voixMax1 = "+res.group(3))

      # Voix Supp
      DebugLog("\n_______________________________________________\nVoix Supp\n")
      DebugLog("-----------------------------------------------\n")
#      r = re.compile("vous reste %s de communication .* votre option (\d+).+?heures d'appels" % self.hmReS, re.DOTALL)
      r = re.compile("vous reste <b>((?:\d{1,2}\s*[Hh])?\s*(?:\d{1,3}\s*(?:mi?n)?))</b> de communication dans le cadre de votre option (\d+\s?h)eures d'appels", re.DOTALL)
      res = r.search(datas)
      if res:
         voixSuppMax = res.group(2)
         DebugLog("voixSupp = "+res.group(1))
         DebugLog("voixSuppMax = "+voixSuppMax)
         self.model.SetConsoVoixSuppMax( voixSuppMax+"00" )
         self.model.SetConsoVoixSupp( res.group(1), invert=True )
            
      # Date de dernière communication
      DebugLog("\n_______________________________________________\nDerniere communication\n")
      DebugLog("-----------------------------------------------\n")
      r = re.compile("prise\sen\scompte.*Le\s+(\d{1,2}.*?\d{2,4}).*?(\d{1,2}).*?(\d{1,2})\s*minutes", re.DOTALL)
      res = r.search(datas)
      if res:
         DebugLog(res.groups())
         date = res.group(1)
         time = "%sh%.2d" % ( res.group(2), int(res.group(3)) )
         self.model.lastTime = "Ma consommation au "+date + " &agrave; "+ time
         DebugLog("last="+self.model.lastTime)
         
      # Date de prochaine facture
      DebugLog("\n_______________________________________________\nProchaine facture\n")
      DebugLog("-----------------------------------------------\n")
      res = re.search("avant le.+?(\d{1,2}\s*.+?\d{2,4})", datas)
      if res:
         self.model.facture = "Prochaine facture : "+res.group(1)
         self.model.facture = self.model.facture.replace("\n", " ")
         DebugLog("facture="+self.model.facture)
         
      # Report des minutes
      DebugLog("\n_______________________________________________\nReport des minutes\n")
      DebugLog("-----------------------------------------------\n")
      r = re.compile("Report Forfait %s de communication"%self.hmReS, re.DOTALL)
      res = r.search(datas)
      if res:
         DebugLog("report = "+res.group(1))
         self.model.SetReport( res.group(1) )

      # Hors forfait
      DebugLog("\n_______________________________________________\nHors forfait\n")
      DebugLog("-----------------------------------------------\n")
      r = re.compile("hors forfait.*?(\d+[.,]?\d*)\s.\sTTC", re.DOTALL)
      res = r.search(datas)
      if res:
         self.model.SetConsoHorsForfait( res.group(1) )
         DebugLog("hforfait = "+res.group(1))

      # SMS
      DebugLog("\n_______________________________________________\nSMS\n")
      DebugLog("-----------------------------------------------\n")
      self.model.sms = "0"
